/*
 * Arguments.java
 *  
 * Copyright 2007 C.A. Meijer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hammingweight.hammockmaker;

/**
 * This class models the arguments of a method.
 * 
 * @author C.A. Meijer
 */
public class Arguments {

	/**
	 * Variable names in code generated by the HammockMaker program start with
	 * this prefix.
	 */
	public static final String VAR_ARG_NAME_PREFIX = "arg";

	/**
	 * The classes of a method's arguments.
	 */
	private Class<?>[] classArgs;

	/**
	 * Constructor.
	 * 
	 * @param classArgs
	 *            The classes of the arguments.
	 */
	public Arguments(Class<?>[] classArgs) {
		this.classArgs = classArgs;
	}

	/**
	 * 
	 * @return The number of arguments that the method takes.
	 */
	public int getNumberOfArgs() {
		return this.classArgs.length;
	}

	/**
	 * 
	 * @return A string of comma separated variables representing the method's
	 *         arguments.
	 */
	public String getArgList() {
		String s = "";
		for (int i = 0; i < this.getNumberOfArgs(); i++) {
			s += ClassWrapper.classToString(classArgs[i]);
			s += " " + VAR_ARG_NAME_PREFIX + Integer.toString(i);
			if (i != this.getNumberOfArgs() - 1) {
				s += ", ";
			}
		}

		return s;
	}

	/**
	 * This method returns the class of the i-th argument for the method.
	 * 
	 * @param i
	 *            The argument of interest.
	 * 
	 * @return A string indicating the class of the argument.
	 */
	public String getObjectClass(int i) {
		if (!this.classArgs[i].isPrimitive()) {
			return ClassWrapper.classToString(this.classArgs[i]) + ".class";
		}

		Class<?> c = this.classArgs[i];
		String type = ClassWrapper.getWrapperForPrimitiveType(c);
		return type + ".class";
	}

	/**
	 * This method returns the i-th argument variable.
	 * 
	 * @param i
	 *            The argument of interest.
	 * 
	 * @return A string representing the i-th variable to be passed to an
	 *         invocation handler.
	 */
	public String getObjectArg(int i) {
		if (!this.classArgs[i].isPrimitive()) {
			return VAR_ARG_NAME_PREFIX + Integer.toString(i);
		}

		Class<?> clazz = this.classArgs[i];
		String type = ClassWrapper.getWrapperForPrimitiveType(clazz);
		return "new " + type + "(" + VAR_ARG_NAME_PREFIX + Integer.toString(i)
				+ ")";
	}
}
